// =============================================================================
//  PN-Triangles Demo - ported to Delphi 2003 by Sascha Willems
//  visit http://www.delphigl.de for more
//
//  And also let me know when you use this code in your projects
//
//  This is a simple port of the PN-Triangles-Demo from ATI which tesselates
//  a very lowpolymodel to a higherpolymodel using the simple
//  GL_PN_TRIANGLES_ATI-Extension...Tesselation has never been easier :-)
//
//  For more see :
//   http://www.ati.com/developer/sdk/RadeonSDK/Html/Info/ati_pn_triangles.txt
//   http://www.ati.com/developer/sdk/RadeonSDK/Html/Samples/OpenGL/PNTriangles.html
//
//  Graphicscards supporting this extension :
//
//    Radeon8500 Series
//    Radeon9x00 Series
//
//  If you have one of these, and it won't tesselate the mesh, then you'll have
//  to enable TRUFORM in the opengl-tab of your driver.This seems to be disabled
//  by default on never drivers/boards...
//
// =============================================================================

unit PNTriangles1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, OpenGL12, ExtCtrls, AppEvnts, glExtensions, StdCtrls;

type
  TForm1 = class(TForm)
    ApplicationEvents: TApplicationEvents;
    FrameTimer: TTimer;
    StatPanel: TPanel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ApplicationEventsIdle(Sender: TObject; var Done: Boolean);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure FrameTimerTimer(Sender: TObject);
  private
    { Private-Deklarationen }
  public
   RC                 : HGLRC;
   DC                 : HDC;
   UsePN              : Boolean;
   WireFrame          : Boolean;
   PNTesselationLevel : Integer;
   MaxPNTessLevel     : Integer;
   Frames             : Integer;
   procedure Draw;
  end;

var
  Form1: TForm1;

const
 Title    = 'PNTriangles-Demo 2003 by Sascha Willems - www.delphigl.de';

 NumVerts = 18;
 NumTris  = 32;

 Indices  : array[0..NumTris*3-1] of TGLUShort = (0,1,2,    1,3,4,    1,4,2,    2,4,5,   0,2,6,    2,5,7,
                                                  2,7,6,    6,7,8,    0,6,9,    6,8,10,  6,10,9,   9,10,11,
                                                  0,9,1,    9,11,12,  9,12,1,   1,12,3,  13,14,15, 14,5,4,
                                                  14,4,15,  15,4,3,   13,16,14, 16,8,7,  16,7,14,  14,7,5,
                                                  13,17,16, 17,11,10, 17,10,16, 16,10,8, 13,15,17, 15,3,12,
                                                  15,12,17, 17,12,11);
 Vertices : array[0..NumVerts*3-1] of TGLFloat = (0,0,37.9,26.8,0,26.8,0,26.8,26.8,37.9,0,0,26.8,26.8,0,0,37.9,0,
                                                  -26.8,0,26.8,-26.8,26.8,0,-37.9,0,0,0,-26.8,26.8,-26.8,-26.8,0,
                                                  0,-37.9,0,26.8,-26.8,0,0,0,-37.9,0,26.8,-26.8,26.8,0,-26.8,
                                                  -26.8,0,-26.8,0,-26.8,-26.8);
 Normals  : array[0..NumVerts*3-1] of TGLFLoat = (0,0,1,0.7,0,0.7,0,0.7,0.7,1,0,0,0.7,0.7,0,0,1,0,-0.7,0,0.7,
                                                  -0.7,0.7,0,-1,0,0,-0,-0.7,0.7,-0.7,-0.7,0,0,-1,0,0.7,-0.7,0,
                                                  0,0,-1,0,0.7,-0.7,0.7,0,-0.7,-0.7,0,-0.7,-0,-0.7,-0.7);
implementation

{$R *.dfm}

procedure TForm1.FormCreate(Sender: TObject);
var
 DummyPal : HPalette;
begin
DC := GetDC(Handle);
RC := CreateRenderingContext(DC, [opDoubleBuffered], 32, 32, 0, 0, 0, 0, DummyPal);
ActivateRenderingContext(DC, RC);
glEnable(GL_DEPTH_TEST);
glDepthFunc(GL_LEQUAL);
glClearColor(0.3, 0.3, 0.3, 1);
UsePN              := True;
PNTesselationLevel := 1;
// Funktionspointer laden
glPNTrianglesiATI := wglGetProcAddress('glPNTrianglesiATI');
if not Assigned(glPNTrianglesiATI) then
 begin
 ShowMessage('GL_PN_TRIANGLES_ATI not supported by Hardware!');
 halt(0);
 end;
// Maximalen PN-Tesselationslevel herausfinden
glGetIntegerV(GL_MAX_PN_TRIANGLES_TESSELATION_LEVEL_ATI, @MaxPNTessLevel);
end;

procedure TForm1.Draw;
const
 Light0 : array[0..3] of TGLFloat = (1,-1,1,0);
 Light1 : array[0..3] of TGLFloat = (1,1,1,0);
 Light2 : array[0..3] of TGLFloat = (-1,1,1,0);
 Color1 : array[0..3] of TGLFloat = (0.6,0.3,0.6,1);
 Color2 : array[0..3] of TGLFloat = (0.3,0.6,0.6,1);
 Spec   : array[0..3] of TGLFloat = (1, 1, 1, 1);
 White  : array[0..3] of TGLFloat = (0.8, 0.8, 0.8, 1);
 Blue   : array[0..3] of TGLFloat = (0.1, 0.1, 1, 1);
begin
glMatrixMode(GL_PROJECTION);
glLoadIdentity;
gluPerspective(60, ClientWidth/ClientHeight, 1, 512);
glMatrixMode(GL_MODELVIEW);
glLoadIdentity;
glClear(GL_DEPTH_BUFFER_BIT or GL_COLOR_BUFFER_BIT);

if WireFrame then
 glPolygonMode(GL_FRONT_AND_BACK, GL_LINE)
else
 glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);

glLightfv(GL_LIGHT0, GL_POSITION, @Light0);
glLightfv(GL_LIGHT1, GL_POSITION, @Light1);
glLightfv(GL_LIGHT2, GL_POSITION, @Light2);
glLightfv(GL_LIGHT1, GL_DIFFUSE, @Color1);
glLightfv(GL_LIGHT1, GL_SPECULAR, @Color1);
glLightfv(GL_LIGHT2, GL_DIFFUSE, @Color2);
glLightfv(GL_LIGHT2, GL_SPECULAR, @Color2);
glEnable(GL_LIGHTING);
glEnable(GL_LIGHT0);
glEnable(GL_LIGHT1);
glEnable(GL_LIGHT2);

glColor3f(1,1,1);
glMaterialfv(GL_FRONT, GL_SPECULAR, @Spec);
glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, @White);
glMaterialf(GL_FRONT, GL_SHININESS, 32);

glEnableClientState(GL_VERTEX_ARRAY);
glVertexPointer(3, GL_FLOAT, 0, @Vertices);

glEnableClientState(GL_NORMAL_ARRAY);
glNormalPointer(GL_FLOAT, 0, @Normals);


glTranslatef(0, 0, -100);

if UsePN then
 begin
 glEnable(GL_PN_TRIANGLES_ATI);
 glPNTrianglesiATI(GL_PN_TRIANGLES_TESSELATION_LEVEL_ATI, PNTesselationLevel);
 end;
glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, @Blue);
glDisable(GL_BLEND);
glEnable(GL_CULL_FACE);
glDrawElements(GL_TRIANGLES, NumTris*3, GL_UNSIGNED_SHORT, @Indices);

glDisable(GL_PN_TRIANGLES_ATI);

SwapBuffers(DC);
inc(Frames);
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
DeActivateRenderingContext;
wglDeleteContext(RC);
ReleaseDC(Handle, DC);
end;

procedure TForm1.ApplicationEventsIdle(Sender: TObject; var Done: Boolean);
begin
Draw;
Done := False;
end;

procedure TForm1.FormKeyPress(Sender: TObject; var Key: Char);
begin
case Key of
 'p' : UsePN := not UsePN;
 'w' : WireFrame := not WireFrame;
 'l' : if PNTesselationLevel < MaxPNTessLevel then
        inc(PNTesselationLevel)
       else
        PNTesselationLevel := 1;
end;
end;

function BoolToStr(pBool : Boolean;pTrue, pFalse : String) : String;
begin
case pBool of
 True  : Result := pTrue;
 False : Result := pFalse;
end;
end;

procedure TForm1.FrameTimerTimer(Sender: TObject);
begin
StatPanel.Caption := 'PN-Tesselation = '+BoolToStr(UsePN, 'on', 'off')+'   '+
                     'PN-Level = '+IntToStr(PNTesselationLevel)+'/'+IntToStr(MaxPNTessLevel)+'   '+
                     'FPS = '+IntToStr(Frames);
Frames := 0;
FrameTimer.Interval := 1000;
end;

end.
